#ifndef _DXAPPLICATION_H
#define _DXAPPLICATION_H

#include "stdinclude.h"
#include <windows.h>
#include <d3d9.h>
#include <d3dx9.h>

namespace MyEngine
{
	class SceneManager;
	class Model;

	class DXApplication
	{
	public:
		static DXApplication* Instance();
		static void DeleteInstance();

	private:
		DXApplication();
		DXApplication(const DXApplication&);
		~DXApplication();
		DXApplication& operator=(const DXApplication&);

	public:
		bool Initialize(HINSTANCE hInstance);
		bool DoConfig(HINSTANCE hInstance);
		int Run();

		LRESULT WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);
		static LRESULT CALLBACK WndProcWrapper(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam);

	private:
		void readSettings();
		bool initWindow();
		bool initInput();
		bool initD3D();
		bool findReqAdapterMode(D3DFORMAT fmt, D3DDISPLAYMODE* pDisplayMode);
		bool initFont();
		bool initScene();

		bool loadModel(const TString& strFilename);
		bool loadModels();
		Model* getModelById(const TString& strId);

		bool isKeyDown(char key) { return m_aKeyboardState[key]; }
		void processKeyboard();

		void freeResources();
		void restoreResources();
		void handleLostDevice();

		void handleMouseMove(int x, int y);

		void makeStep();
		void updateScene(uint ms);
		void renderScene();

	private:
		bool m_bDeviceLost;
		bool m_aKeyboardState[256];

		ModelMap m_models;
		SceneManager* m_pSceneManager;

	private:
		static DXApplication* m_pInstance;
		
		TString m_strWindowTitle;
		TString m_strWindowClass;

		HINSTANCE m_hInstance;
		HWND m_hWnd;

		LPDIRECT3D9 m_pd3dObject;
		LPDIRECT3DDEVICE9 m_pd3dDevice;
		D3DPRESENT_PARAMETERS m_d3dPP;
		LPD3DXFONT m_pd3dFont;

		uint m_nWidth;
		uint m_nHeight;
		uint m_nRefreshRate;

		uint m_nAppStartTime;
		uint m_nPrevFrameTime;
		uint m_nNextFrameTime;
		uint m_nTimeStep;

		float m_fFps;
		uint m_nFrameCount;
		uint m_nFpsStartTime;
	};
}

#endif // _DXAPPLICATION_H
