#include "SceneManager.h"
#include "Camera.h"
#include "Entity.h"

namespace MyEngine
{
	SceneManager::SceneManager(LPDIRECT3DDEVICE9 pDevice)
		: m_pDevice(pDevice)
	{
		assert(pDevice != NULL);
	}

	SceneManager::~SceneManager()
	{
		EntityList::iterator i = m_dynamicObjects.begin();

		while(i != m_dynamicObjects.end())
		{
			delete *i;
			*i = NULL;
			i++;
		}
	}

	void SceneManager::UpdateScene(uint ms)
	{
		float dt = ms / 1000.0F;

		EntityList::iterator i = m_dynamicObjects.begin();

		while(i != m_dynamicObjects.end())
		{
			(*i)->Update(dt);
			i++;
		}

		m_pCamera->Update(dt);
		m_pSkybox->SetPosition(m_pPlayer->Position());
	}

	void SceneManager::RenderScene()
	{
		// /////////////////////////////////////////////////////////////////
		// / Apply the view matrix.
		// /////////////////////////////////////////////////////////////////
		D3DXMATRIX view = m_pCamera->ViewMatrix();
		m_pDevice->SetTransform(D3DTS_VIEW, &view);

		// /////////////////////////////////////////////////////////////////
		// / Apply the projection matrix.
		// /////////////////////////////////////////////////////////////////
		D3DXMATRIX projection = m_pCamera->ProjectionMatrix();
		m_pDevice->SetTransform(D3DTS_PROJECTION, &projection);

		// /////////////////////////////////////////////////////////////////
		// / Set texture sampler state.
		// /////////////////////////////////////////////////////////////////
		m_pDevice->SetSamplerState(0, D3DSAMP_MINFILTER, D3DTEXF_LINEAR);
		m_pDevice->SetSamplerState(0, D3DSAMP_MAGFILTER, D3DTEXF_LINEAR);
		m_pDevice->SetSamplerState(0, D3DSAMP_MIPFILTER, D3DTEXF_LINEAR);

		// /////////////////////////////////////////////////////////////////
		// / Render the skybox first.
		// /////////////////////////////////////////////////////////////////
		m_pDevice->SetRenderState(D3DRS_LIGHTING, FALSE);
		m_pDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
		m_pDevice->SetRenderState(D3DRS_ZWRITEENABLE, FALSE);

		m_pSkybox->Render();

		// /////////////////////////////////////////////////////////////////
		// / Render the rest of renderable entities.
		// /////////////////////////////////////////////////////////////////
		m_pDevice->SetRenderState(D3DRS_LIGHTING, TRUE);
		m_pDevice->SetRenderState(D3DRS_AMBIENT, D3DCOLOR_XRGB(255,255,255));
		m_pDevice->SetRenderState(D3DRS_ZENABLE, D3DZB_TRUE);
		m_pDevice->SetRenderState(D3DRS_ZWRITEENABLE, TRUE);

		EntityList::iterator i = m_dynamicObjects.begin();

		while(i != m_dynamicObjects.end())
		{
			(*i)->Render();
			i++;
		}
	}
}
