#include <windows.h>
#include <commctrl.h>
#include <tchar.h>
#include <d3d9.h>
#include <vector>
#include <map>
#include <sstream>
#include <fstream>

// IDs of window controls
#define IDC_RESOLUTION	102		// combo box
#define IDC_REFRESH		103		// combo box
#define IDC_OK			104		// button
#define IDC_CANCEL		105		// button

struct Resolution;

// Some typedefs to make typing easier
typedef unsigned int uint;
typedef std::basic_string<TCHAR> TString;
typedef std::basic_ostringstream<TCHAR> TOStringStream;
typedef std::basic_ofstream<TCHAR> TOFileStream;
typedef std::map<TString, Resolution> ResolutionMap;

struct Resolution
{
	uint Width;
	uint Height;
	std::vector<uint> RefreshRates;
};

// Classic Win32 stuff
HINSTANCE g_hInstance = NULL;
TCHAR g_szWindowTitle[128] = _T("Settings");
TCHAR g_szWindowClass[128] = _T("CFGWINDOWCLASS");

// Function prototypes
void EnumerateAdapterModes();
void CreateChildControls(HWND);
void PopulateRefreshRates(HWND, TString);
void SaveDataToFile(TString, int);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);

// Global variables
ResolutionMap g_modes;

int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
				   LPSTR lpCmdLine, int nCmdShow)
{
	UNREFERENCED_PARAMETER(hPrevInstance);
	UNREFERENCED_PARAMETER(lpCmdLine);

	g_hInstance = hInstance;

	EnumerateAdapterModes();

	// This is necessary to enable XP visual styles
	INITCOMMONCONTROLSEX commCtrl;
	commCtrl.dwSize = sizeof(INITCOMMONCONTROLSEX);
	InitCommonControlsEx(&commCtrl);

	WNDCLASSEX wcex;
	wcex.cbSize = sizeof(WNDCLASSEX);
	wcex.style = CS_HREDRAW | CS_VREDRAW;
	wcex.lpfnWndProc = WndProc;
	wcex.cbClsExtra = 0;
	wcex.cbWndExtra = 0;
	wcex.hInstance = g_hInstance;
	wcex.hIcon = NULL;
	wcex.hCursor = LoadCursor(NULL, IDC_ARROW);
	wcex.hbrBackground = (HBRUSH)(COLOR_BTNFACE+1);
	wcex.lpszMenuName = NULL;
	wcex.lpszClassName = g_szWindowClass;
	wcex.hIconSm = NULL;

	if(RegisterClassEx(&wcex) == 0)
		return 0;

	// Center window horizontally and vertically
	int nWindowWidth = 225;
	int nWindowHeight = 125;
	int nX = (GetSystemMetrics(SM_CXSCREEN) - nWindowWidth) / 2;
	int nY = (GetSystemMetrics(SM_CYSCREEN) - nWindowHeight) / 2;

	HWND hWnd = CreateWindow(g_szWindowClass, g_szWindowTitle, WS_POPUPWINDOW | WS_CAPTION,
		nX, nY, nWindowWidth, nWindowHeight, NULL, NULL, g_hInstance, NULL);

	if(hWnd == NULL)
		return 0;

	ShowWindow(hWnd, nCmdShow);
	UpdateWindow(hWnd);

	MSG msg;
	while(GetMessage(&msg, NULL, 0, 0))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}

	return (int)msg.wParam;
}

LRESULT CALLBACK WndProc(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch(uMsg)
	{
		case WM_CREATE:
			CreateChildControls(hWnd);
			break;

		case WM_COMMAND:
		{
			switch(HIWORD(wParam))
			{
				case BN_CLICKED:
				{
					switch(LOWORD(wParam))
					{
						case IDC_OK:
						{
							TCHAR buffer[64];
							int index1 = SendDlgItemMessage(hWnd, IDC_RESOLUTION, CB_GETCURSEL, (WPARAM)0, 0L);
							SendDlgItemMessage(hWnd, IDC_RESOLUTION, CB_GETLBTEXT, (WPARAM)index1, (LPARAM)buffer);
							int index2 = SendDlgItemMessage(hWnd, IDC_REFRESH, CB_GETCURSEL, (WPARAM)0, 0L);

							SaveDataToFile(buffer, index2);
							PostQuitMessage(0);
							break;
						}

						case IDC_CANCEL:
							PostQuitMessage(0);
							break;
					}

					break;
				}

				case CBN_SELCHANGE:
				{
					if(LOWORD(wParam) == IDC_RESOLUTION)
					{
						TCHAR buffer[64];
						int index = SendDlgItemMessage(hWnd, IDC_RESOLUTION, CB_GETCURSEL, (WPARAM)0, 0L);
						SendDlgItemMessage(hWnd, IDC_RESOLUTION, CB_GETLBTEXT, (WPARAM)index, (LPARAM)buffer);

						PopulateRefreshRates(hWnd, buffer);
					}

					break;
				}
			}

			break;
		}

		case WM_DESTROY:
		{
			PostQuitMessage(0);
			break;
		}

		default:
			return DefWindowProc(hWnd, uMsg, wParam, lParam);
	}

	return 0;
}

void CreateChildControls(HWND hWndParent)
{
	// Resolutions
	HWND hCmbResolution = CreateWindow(_T("COMBOBOX"), NULL, WS_CHILD | WS_VISIBLE | WS_VSCROLL |
		CBS_DROPDOWNLIST, 15, 15, 100, 200, hWndParent, (HMENU)IDC_RESOLUTION, g_hInstance, NULL);

	// Refresh rates
	HWND hCmbRefresh = CreateWindow(_T("COMBOBOX"), NULL, WS_CHILD | WS_VISIBLE | WS_VSCROLL |
		CBS_DROPDOWNLIST, 130, 15, 75, 100, hWndParent, (HMENU)IDC_REFRESH, g_hInstance, NULL);

	// Populate both combo boxes with data
	ResolutionMap::iterator it = g_modes.begin();
	while(it != g_modes.end())
	{
		SendMessage(hCmbResolution, CB_ADDSTRING, 0, (LPARAM)it->first.c_str());
		it++;
	}

	SendMessage(hCmbResolution, CB_SETCURSEL, 0, 0L);
	PopulateRefreshRates(hWndParent, g_modes.begin()->first);

	// OK button
	HWND hBtnOK = CreateWindow(_T("BUTTON"), _T("OK"), WS_CHILD | WS_VISIBLE,
		155, 55, 50, 25, hWndParent, (HMENU)IDC_OK, g_hInstance, NULL);

	// Cancel button
	HWND hBtnCancel = CreateWindow(_T("BUTTON"), _T("Cancel"), WS_CHILD | WS_VISIBLE,
		15, 55, 50, 25, hWndParent, (HMENU)IDC_CANCEL, g_hInstance, NULL);

	// Default font
	HFONT hFont = (HFONT)GetStockObject(DEFAULT_GUI_FONT);
	SendMessage(hCmbResolution, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(FALSE, 0));
	SendMessage(hCmbRefresh, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(FALSE, 0));
	SendMessage(hBtnOK, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(FALSE, 0));
	SendMessage(hBtnCancel, WM_SETFONT, (WPARAM)hFont, MAKELPARAM(FALSE, 0));
}

// ***********************************************************************************
// * Populates the second combo box with refresh rates based on resolution selected
// * from the first combo box.
// ***********************************************************************************
void PopulateRefreshRates(HWND hWndParent, TString strResolution)
{
	SendDlgItemMessage(hWndParent, IDC_REFRESH, CB_RESETCONTENT, 0, 0L);

	// Find refresh rates available for selected resolution
	ResolutionMap::iterator it = g_modes.find(strResolution);
	if(it != g_modes.end())
	{
		for(uint i=0; i < it->second.RefreshRates.size(); ++i)
		{
			TOStringStream rr;
			rr << it->second.RefreshRates[i] << " Hz";

			SendDlgItemMessage(hWndParent, IDC_REFRESH, CB_ADDSTRING, 0,
				(LPARAM)rr.str().c_str());
		}

		// Automatically select the first refresh rate
		SendDlgItemMessage(hWndParent, IDC_REFRESH, CB_SETCURSEL, 0, 0L);
	}
}

// ***********************************************************************************
// * Saves selected adapter mode to a file.
// ***********************************************************************************
void SaveDataToFile(TString strResolution, int nRefreshRateIndex)
{
	ResolutionMap::iterator it = g_modes.find(strResolution);
	if(it != g_modes.end())
	{
		TOFileStream file;
		file.open(_T("settings.ini"), std::ios_base::out | std::ios_base::trunc);

		if(file)
		{
			file << it->second.Width << std::endl;
			file << it->second.Height << std::endl;
			file << it->second.RefreshRates[nRefreshRateIndex] << std::endl;
			file.close();
		}
	}
}

// ***********************************************************************************
// * Uses Direct3D to enumerate available adapter modes and saves them inside a
// * global variable g_modes.
// ***********************************************************************************
void EnumerateAdapterModes()
{
	LPDIRECT3D9 pd3dObject = Direct3DCreate9(D3D_SDK_VERSION);

	if(pd3dObject == NULL)
		return;

	D3DFORMAT fmt = D3DFMT_X8R8G8B8;

	uint nModeCount = pd3dObject->GetAdapterModeCount(D3DADAPTER_DEFAULT, fmt);

	for(uint i=0; i < nModeCount; ++i)
	{
		D3DDISPLAYMODE dm;
		HRESULT hr = pd3dObject->EnumAdapterModes(D3DADAPTER_DEFAULT, fmt, i, &dm);

		if(SUCCEEDED(hr))
		{
			// Construct a key for map item
			TOStringStream key;
			key << dm.Width << " x " << dm.Height;

			// See if item with that key already exists
			ResolutionMap::iterator it = g_modes.find(key.str());

			if(it != g_modes.end())
			{
				it->second.RefreshRates.push_back(dm.RefreshRate);
			}
			else
			{
				Resolution r;
				r.Width = dm.Width;
				r.Height = dm.Height;
				r.RefreshRates.push_back(dm.RefreshRate);

				g_modes[key.str()] = r;
			}
		}
	}

	if(pd3dObject != NULL)
	{
		pd3dObject->Release();
		pd3dObject = NULL;
	}
}
